@@ -306,7 +306,7 @@ htmlIsBooleanAttr(const xmlChar *name)
     int i = 0;
 
     while (htmlBooleanAttrs[i] != NULL) {
-        if (xmlStrcmp((const xmlChar *)htmlBooleanAttrs[i], name) == 0)
+        if (xmlStrcasecmp((const xmlChar *)htmlBooleanAttrs[i], name) == 0)
             return 1;
         i++;
     }
